@@ -4,18 +4,24 @@ use strict;
 use warnings;
 
 our $VERSION;
-$VERSION = '0.05';
+$VERSION = '0.06';
 
 use Net::DNS qw(:DEFAULT);
 use base     qw(Exporter Net::DNS);
 
 our @EXPORT = @Net::DNS::EXPORT;
 
-use Carp;
+use IO::Select;
 use IO::Socket;
 use Socket qw(pack_ip_mreq pack_ipv6_mreq inet_pton
-		IPPROTO_IP IP_ADD_MEMBERSHIP IP_MULTICAST_TTL
-		IPPROTO_IPV6 IPV6_ADD_MEMBERSHIP IPV6_MULTICAST_HOPS);
+		IPPROTO_IP IPPROTO_IPV6 IP_MULTICAST_TTL IPV6_MULTICAST_HOPS
+		IP_ADD_MEMBERSHIP IPV6_ADD_MEMBERSHIP IPV6_JOIN_GROUP);
+
+use constant REUSEADDR => Socket->can('SO_REUSEADDR');
+use constant REUSEPORT => Socket->can('SO_REUSEPORT');
+use constant ADDMEMBER => Socket->can('IPV6_ADD_MEMBERSHIP');
+use constant JOINGROUP => Socket->can('IPV6_JOIN_GROUP');
+
 
 =head1 NAME
 
@@ -80,16 +86,17 @@ sub Net::DNS::Resolver::bgsend {
 	my $port   = $self->port($multicast_port);
 	foreach my $ip ( $self->nameservers(@multicast_group) ) {
 		my $socket = $self->_create_udp_socket($ip);
-		$socket->setsockopt( SOL_SOCKET, SO_REUSEADDR, 1 );
-		$socket->setsockopt( SOL_SOCKET, SO_REUSEPORT, 1 );
+		$socket->setsockopt( SOL_SOCKET, SO_REUSEADDR, 1 ) if REUSEADDR;
+		$socket->setsockopt( SOL_SOCKET, SO_REUSEPORT, 1 ) if REUSEPORT;
 		if ( $socket->sockdomain() == AF_INET6 ) {
 			my $ip_mreq = pack_ipv6_mreq( inet_pton( AF_INET6, $ip ), 0 );
-			$socket->setsockopt( IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, $ip_mreq );
+			$socket->setsockopt( IPPROTO_IPV6, IPV6_ADD_MEMBERSHIP, $ip_mreq ) if ADDMEMBER;
+			$socket->setsockopt( IPPROTO_IPV6, IPV6_JOIN_GROUP,	$ip_mreq ) if JOINGROUP;
 			$socket->setsockopt( IPPROTO_IPV6, IPV6_MULTICAST_HOPS, 1 );
 		} else {
 			my $ip_mreq = pack_ip_mreq( inet_aton($ip), INADDR_ANY );
 			$socket->setsockopt( IPPROTO_IP, IP_ADD_MEMBERSHIP, $ip_mreq );
-			$socket->setsockopt( IPPROTO_IP, IP_MULTICAST_TTL, 1 );
+			$socket->setsockopt( IPPROTO_IP, IP_MULTICAST_TTL,  1 );
 		}
 
 		my $expire = time() + $multicast_timeout;
